@@ -43,6 +43,9 @@ import netCDF4
 
 import utils
 
+def vsnmsg(version): 
+    return "This file was generated using WHIPS v{0}".format(version)
+
 def ValidOutfuncs():
     '''Return a list of valid output function names'''
     currentModule = sys.modules[__name__]
@@ -53,7 +56,7 @@ class out_func:
     '''Abstract class to for <>_out_geo classes'''
     def __init__(self, parmDict=None):
         self.parmDict = parmDict
-    def __call__(self, map_geo, griddef, outfilenames, verbose):
+    def __call__(self, map_geo, griddef, outfilenames, verbose, version):
         raise NotImplementedError
     @staticmethod
     def parm_list():
@@ -73,8 +76,9 @@ class invalidPixCeption(Exception):
 tai93conv = lambda(timestring):utils.timestr_to_nsecs(timestring, 
                                '00:00:00_01-01-1993', '%H:%M:%S_%m-%d-%Y')
 
+# currently borked.  No immediate plans to fix
 #class OMNO2e_wght_avg_out_func(out_func):
-class OMONO2e_wght_avg_BORKED(out_func): 
+class OMNO2e_wght_avg_BORKED(out_func): 
     '''
     Weighted avg based on OMNO2e algorithm
 
@@ -146,7 +150,10 @@ class OMONO2e_wght_avg_BORKED(out_func):
                              'netCDF file.  This value will replace any missing '\
                              ' or invalid output values','decimal')}
     
-    def __call__(self, maps, griddef, outfilename, verbose):
+    # userKeys not necessary, so 'filler' field used instead
+    __userKeys__ = "filetype"
+
+    def __call__(self, maps, griddef, outfilename, verbose, version):
 
         # function is broken until it can be refactored such that
         # _OMNO2e_func doesn't require totFlag.  Needs to have pixel
@@ -416,8 +423,10 @@ class OMNO2e_netCDF_avg_out_func(out_func):
                 'fillVal' : ('The value to use as a fill value in the output ' \
                              'netCDF file.  This value will replace any missing '\
                              'or invalid output values','decimal')}
+    # variable signifying which list is to act as the master list index
+    __userKeys__ = "inFieldNames"
 
-    def __call__(self, maps, griddef, outfilename, verbose):
+    def __call__(self, maps, griddef, outfilename, verbose, version):
         '''Write out a weighted-average file in netcdf format.'''
         #Make sure non-string parameters are in the correct format
         dimsizes = self.parmDict['extraDimSize']
@@ -568,6 +577,7 @@ class OMNO2e_netCDF_avg_out_func(out_func):
         outFid.createDimension('row', nRows)
         outFid.createDimension('col', nCols)
         # write global attributes
+        setattr(outFid, 'Version', vsnmsg(version))
         setattr(outFid, 'File_start_time', utils.nsecs_to_timestr(self.parmDict['timeStart'], '00:00:00 01-01-1993'))
         setattr(outFid, 'File_end_time', utils.nsecs_to_timestr(self.parmDict['timeStop'], '00:00:00 01-01-1993'))
         setattr(outFid, 'Max_valid_cloud_fraction', self.parmDict['cloudFractUpperCutoff'])
@@ -760,6 +770,7 @@ class wght_avg_netCDF(out_func):
             is safe to use both get and get_cm functions within this function -
             it is guaranteed to be called within a context manager.
     '''
+
     def __init__(self, parmDict=None):
         # call ancestor method
         out_func.__init__(self, parmDict)
@@ -780,9 +791,9 @@ class wght_avg_netCDF(out_func):
         # confirm that each tuple is the same size
         tupsMatch = [len(l) == len(s) for (l,s) in izip(labelTups, sizeTups)]
         if not all(tupsMatch):
-            misMatch = [l+' does not match '+s for (l,s,m) in izip(labelTups,sizeTups,tupsMatch) if not m]
+            misMatch = [l + ' does not match ' + s for (l,s,m) in izip(labelTups,sizeTups,tupsMatch) if not m]
             msg = "All tuple-like strings must correspond to tuples of corresponding size. " \
-                "The following sets do not correspond: "+' '.join(misMatch)
+                "The following sets do not correspond: \n" + '\n'.join(misMatch)
             raise ValueError(msg)
         # convert sizes to integers
         try:
@@ -818,7 +829,7 @@ class wght_avg_netCDF(out_func):
         for key in lists:
             self.parmDict[key] = dict(zip(inFnames, self.parmDict[key]))
         
-    def __call__(self, maps, griddef, outfilename, verbose):
+    def __call__(self, maps, griddef, outfilename, verbose, version):
         '''Write out a weighted-average file in netcdf format.'''
         
         # create a dictionary of numpy arrays that will hold the data for all 
@@ -905,7 +916,11 @@ class wght_avg_netCDF(out_func):
                         # create a slice object that will allow us to broadcast
                         # weights against the values
                         extraDims = self.parmDict['dimSizes'][field]
-                        nExtraDims = len(extraDims)
+                        # handle the special case where we put in 0 for extradims
+                        if len(extraDims) == 1 and extraDims == [0]:
+                            nExtraDims = 0
+                        else:
+                            nExtraDims = len(extraDims)
                         wghtSlice = [Ellipsis]+[numpy.newaxis]*nExtraDims
 
                         # handle special case where there were no pixels in
@@ -953,6 +968,7 @@ class wght_avg_netCDF(out_func):
         outFid.createDimension('col', nCols)
 
         # set global attributes
+        setattr(outFid, 'Version', vsnmsg(version))
         setattr(outFid, 'File_start_time', 
                 utils.nsecs_to_timestr(self.parmDict['timeStart'], 
                                        epoch='00:00:00 01-01-1993', 
@@ -1177,7 +1193,7 @@ class unweighted_filtered_MOPITT_avg_netCDF_out_func(wght_avg_netCDF):
                                      'decimal'),
                 'solZenAng' : ('The name of the field containing the solar' \
                                ' zenith angle in degrees.  { MOPITT - Solar ' \
-                               'Zenith Angle }', 'decimal'),
+                               'Zenith Angle }', None),
                 'dayTime' : ('Boolean variable that indicates ' \
 			     'whether the output file should contain ' \
 			     'values from day or night.  If set to ' \
@@ -1198,6 +1214,8 @@ class unweighted_filtered_MOPITT_avg_netCDF_out_func(wght_avg_netCDF):
                                   'values and std devs) of which we want the ' \
                                   'first slice.\n{ MOPITT - Retrieved CO Mixing '\
                                   'Ratio Profile }', None)}
+    # variable signifying which list is to act as the master list index 
+    __userKeys__ = "inFieldNames"
     def __init__(self, pDict):
         '''Convert input to format of parent input'''
 
