@@ -75,20 +75,53 @@ import pdb
 
 import tables
 import numpy
-
 import pyhdf.HDF
 import pyhdf.V
 import pyhdf.VS
 import pyhdf.SD
 
+import filetypes
 
 def SupportedFileTypes():
     '''Return a list of supported file types'''
-    currentModule = sys.modules[__name__]
-    names = dir(currentModule)
-    return [el[:-5] for el in names if el.endswith("_File")]
+    return [el[:-9] for el in dir(filetypes) if el.endswith("_filetype")]
 
-def get_parser(file, filetype):
+
+def getOrbitNumber(fPath):    
+    '''Takes in the path to a nasa omi hdf file and returns the orbit number'''
+    fid = tables.openFile(fPath)
+    try:
+        node = fid.getNode('/', 'HDFEOS INFORMATION/CoreMetadata')
+    except tables.exceptions.NoSuchNodeError:
+        node = fid.getNode('/', 'HDFEOS INFORMATION/CoreMetadata.0')
+    bigString = str(list(node)[0])
+    strings = bigString.split('\n')
+    for i in range(len(strings)):
+        if 'ORBITNUMBER' in strings[i]:
+            break
+    line = strings[i+3]
+    numArray = [int(el) for el in line.split() if el.isdigit()]
+    fid.close()
+    return numArray[0]
+
+def getLongName(fPath):
+    '''Retrieve the long name of an HDFEOS file'''
+    fid = tables.openFile(fPath)
+    try:
+        node = fid.getNode('/', 'HDFEOS INFORMATION/ArchiveMetadata')
+    except tables.exceptions.NoSuchNodeError:
+        node = fid.getNode('/', 'HDFEOS INFORMATION/ArchiveMetadata.0')
+    bigString = str(list(node)[0])
+    strings = bigString.split('\n')
+    for i in range(len(strings)):
+        if 'LONGNAME' in strings[i]:
+            break
+    line = strings[i+2]
+    chunks = line.split('"')
+    fid.close()
+    return chunks[-2]
+
+def get_parser(file, filetype, parserParms):
     """Retrieve appropriate instantiated parser for a file"""
     # filename = os.path.split(file)[1]
     subclass = '{0}_File'.format(filetype)
@@ -102,7 +135,7 @@ def get_parser(file, filetype):
             extension += i
         else:
             subtype += i
-    return parserClass(file, subtype, extension)
+    return parserClass(file, subtype, extension, **parserParms)
 
 class GeoFile():
     """Provide interface to geofile."""
@@ -538,22 +571,82 @@ class HDFnasaomil2_File(HDFFile):
     """
     Provide interface to NASA OMI L2 product, with pixel corners
     
-    Pixel corners are retrieved from an extra file that must be passed
-    as the extra parameter pixCornerFname.  It is the user's 
-    responsibility to make sure that the pixel corner file matches the
-    data file.  The corners retrieved are for the visible channel used
-    by the NO2 algorithm- using this parser for other products may 
+    Pixel corners are retrieved from an extra file that must be accessible
+    in cornerDir.  The files listed in cornerFileList, if any, will be checked
+    first, followed by any files in cornerDir. If no valid pixel corner file
+    is found that matches the orbit number of the input file, the parser will
+    instantiage but get_geo_corners will fail with an IOError. 
+
+    The corners retrieved are for the visible channel
+    used by the NO2 algorithm- using this parser for other products may 
     require altering the parser to use a different channel if 
     appropriate.
     
     The keys to retrieve variables are the names of the variables within actual
     files.  Note that the NASA product documentation has the wrong name for 
     the "SlantColumnAmountH20Std" variable due to a case typo.
+
+    Does not support the fields "UnpolFldCoefficients" or "SmallPixelRadiance"
+    because the dimensionality of these fields changes between files and there
+    is no way to deal with fields with variable dimension size in the current 
+    framework.
     """
-    def __init__(self, filename, subtype='', extension=None, pixCornerFname=None):
+    
+    OMIAURANO2_FILE_NAME = "OMI/Aura Nitrogen Dioxide (NO2) Total & "\
+                           "Troposph.Column 1-Orbit L2 Swath 13x24km"
+    OMIAURANO2_CORNER_FILE_NAME = "OMI/Aura Global Ground Pixel Corners "\
+                                  "1-Orbit L2 Swath 13x24km"
+
+    def __init__(self, filename, subtype='', extension=None, cornerDir=None,
+                 cornerFileList=None):
         HDFFile.__init__(self, filename, subtype, extension)
-        self.pixCorners = pixCornerFname
+
+        # make sure filename is actually an input file
+        if getLongName(filename) != HDFnasaomil2_File.OMIAURANO2_FILE_NAME:
+            raise IOError('Attempt to read non-NASA OMI L2 file as such.')
         
+        # start by assuming we aren't going to find anything
+        self.pixCorners = None
+
+        # see if the corner directory even exists.  If it doesn't, we obviously can't 
+        # find a corner file
+        if os.path.isdir(cornerDir):
+
+            # convert the corner files into full pathnames
+            # unless we were given null string (signal to search directory)
+            if cornerFileList != ['']:
+                cornerFileList = [os.path.join(cornerDir, f) for f in cornerFileList]
+
+            # get orbit number of file for matching
+            forbitnumber = getOrbitNumber(filename)
+
+            # try using the list
+        
+            for f in cornerFileList:
+                if f != '' and getLongName(f) == HDFnasaomil2_File.OMIAURANO2_CORNER_FILE_NAME:
+                    try:
+                        if getOrbitNumber(f) == forbitnumber:
+                            self.pixCorners = f
+                            break
+                    except:
+                        pass
+
+            # if necessary, search entire corner file directory
+            if self.pixCorners == None:        
+                allPossible = [os.path.join(cornerDir, f) for f in os.listdir(cornerDir)]
+                for f in allPossible:
+                    try:
+                        if tables.isHDF5File(f) \
+                                and getLongName(f) == HDFnasaomil2_File.OMIAURANO2_CORNER_FILE_NAME \
+                                and getOrbitNumber(f) == forbitnumber:
+                            self.pixCorners = f
+                            break
+                    except:
+                        pass
+                
+        if self.pixCorners == None:
+            print "No valid corner file found for {0}.".format(filename)
+            
     __dataPath = '/HDFEOS/SWATHS/ColumnAmountNO2/Data Fields/'
     __geoPath = '/HDFEOS/SWATHS/ColumnAmountNO2/Geolocation Fields/'
     
@@ -619,7 +712,6 @@ class HDFnasaomil2_File(HDFFile):
                     'TerrainReflectivity' : __dataPath+'TerrainReflectivity',
                     'TropFractionUnpolluted' : __dataPath+'TropFractionUnpolluted',
                     'TropFractionUnpollutedStd' : __dataPath+'TropFractionUnpollutedStd',
-                    'UnpolFldCoefficients' : __dataPath+'UnpolFldCoefficients',
                     'UnpolFldLatBandQualityFlags' : __dataPath+'UnpolFldLatBandQualityFlags',
                     'WavelengthRegistrationCheck' : __dataPath+'WavelengthRegistrationCheck',
                     'WavelengthRegistrationCheckStd' : __dataPath+'WavelengthRegistrationCheckStd',
@@ -644,7 +736,6 @@ class HDFnasaomil2_File(HDFFile):
                   'MeasurementQualityFlags' : lambda var, ind: var[ind[0]],
                   'WavelengthRegistrationCheck' : lambda var, ind: var[ind[0], :],
                   'WavelengthRegistrationCheckStd' : lambda var, ind: var[ind[0], :],
-                  'UnpolFldCoefficients' : lambda var, ind: var[:],
                   'UnpolFldLatBandQualityFlags' : lambda var, ind: var[:],
                   'Time' : lambda var, ind: var[ind[0]],
                   'SpacecraftLatitude' : lambda var, ind: var[ind[0]],
@@ -662,7 +753,7 @@ class HDFnasaomil2_File(HDFFile):
         try:
             pxFid = tables.openFile(self.pixCorners)
         except AttributeError:
-            raise IOError('Unable to open pixel corners file.  File may not have been specified.')
+            raise IOError('Unable to open pixel corners file.  Need pixel corners file to use corners')
         try:
             latNode = pxFid.getNode('/', latNodeName)
             lonNode = pxFid.getNode('/', lonNodeName)
@@ -736,7 +827,6 @@ class HDFmopittl2_File(HDF4File):
         '''Overloaded version of get_cm that applied the correct missing value.'''
         return HDF4File.get_cm(self, key, indices, missingValue=-9999.0)
 
-
     def get_geo_centers(self):
         '''Retrieves array of the corners of the pixels'''
         lat = self.get('Latitude').squeeze()
